local ships = {}
local player = {}
local radar = {}
local bullet = {}
local ship = {}
local map = {}
local camera = {}
local trails = {}
local bg = {}
local live = {}
local planes = {}
local plane = {}
local game = {}
local explosions = {}
local explosion = {}
local ui = {}

function love.load()
    game.state = 0
    game.startTime = 5
    game.startTimer = 0
    
    explosion.count = 0
    explosion.image = love.graphics.newImage("ex.png")
    explosion.time = 0.75
    explosion.times = {}
    
    map.width = 2000
    map.height = 1500
    
    bg.image = love.graphics.newImage("bg.png")
    bg.overlayImage = love.graphics.newImage("white.png")
    
    camera.x = 0
    camera.y = 0
    camera.xMax = map.width - love.graphics.getWidth()
    camera.yMax = map.height - love.graphics.getHeight()
    
    radar.angle = 0
    radar.time = 0
    radar.interval = 2
    radar.length = math.sqrt(love.graphics.getWidth() ^ 2 + love.graphics.getHeight() ^ 2)
    radar.trail = {}
    radar.music = love.audio.newSource("radar.wav", "static")
    
    player.x = map.width / 2
    player.y = map.height / 2
    player.angle = 0
    player.speed = 150
    player.turnSpeed = 1
    player.shootInterval = 0.05
    player.shootTime = player.shootInterval
    player.radius = 10
    player.minBulletDetectionRange = 75
    player.maxBulletDetectionRange = 125
    player.minEnemyDetectionRange = 50
    player.maxEnemyDetectionRange = 100
    player.die = false
    player.image = love.graphics.newImage("play.png")
    player.trailCount = 0
    player.trailInterval = 1
    player.trailTimer = 0
    player.maxAmmo = 10
    player.currentAmmo = player.maxAmmo
    player.reloadTime = 3
    player.reloadTimer = 0
    player.lines = {}
    player.explosions = {}
    player.explosionCount = 0
    player.explosionImage = love.graphics.newImage("ex.png")
    player.explosionTime = 0.75
    player.explosionTimes = {}
    player.hitSound = love.audio.newSource("hot.wav", "static")
    
    explodeSound = love.audio.newSource("ex.wav", "static")
    enemyHitSound = love.audio.newSource("hit.wav", "static")
    
    ship.count = 20
    ship.radius = 10
    ship.maxWaitTime = 5;
    ship.movement = 10
    ship.spawnInterval = 5
    ship.minSpawnInterval = 1
    ship.spawnIntervalModifier = 0.5
    ship.spawnTimer = 0
    ship.bulletCount = 0
    ship.shootInterval = 3
    ship.bullets = {}
    ship.bulletSpeed = 225
    ship.bulletRadius = 3
    ship.image = love.graphics.newImage("ship2.png")
    ship.bulletImage = love.graphics.newImage("bul.png")
    for i = 1, ship.count do
        repeat
            ships[i] = {x = love.math.random(0, map.width), y = love.math.random(0, map.height), 
                radarAlpha = 0, waitTime = ship.maxWaitTime, maxWaitTime = ship.maxWaitTime, angle = 0, 
                move = true, shootTimer = love.math.random(0, 5)}
        until math.sqrt((ships[i].x - player.x) ^ 2 + (ships[i].y - player.y) ^ 2) >= 150
    end
    
    plane.count = 0
    plane.wave = 0
    plane.drawWave = false
    plane.waveTime = 1
    plane.waveTimer = 0
    plane.waveRealTime = 0
    plane.nextSpawn = 1
    plane.maxSpawn = 10
    plane.radius = 10
    plane.speed = 100
    plane.turnSpeed = 0.75
    plane.image = love.graphics.newImage("plane2.png")
    plane.bullets = {}
    plane.bulletSpeed = 500
    plane.bulletRadius = 3
    plane.bulletCount = 0
    plane.bulletImage = love.graphics.newImage("eb.png")
    plane.shootDuration = 1
    plane.shootInterval = 0.2
    plane.shootCooldown = 10
    
    bulletList = {prev = nil, next = nil, value = {x = 99999, y = 99999, angle = 0, alpha = 0}}
    bullet.speed = 1000
    bullet.radius = 3
    bullet.minAlphaRange = 300
    bullet.maxAlphaRange = 1500
    bullet.image = love.graphics.newImage("pb.png")
    bullet.sound = love.audio.newSource("shoot.wav", "static")
    
    live.maxLive = 50
    live.count = live.maxLive
    
    ui.leftImage = love.graphics.newImage("ui_l.png")
    ui.rightImage = love.graphics.newImage("ui_r.png")
    ui.middleImage = love.graphics.newImage("ui_m.png")
    
    xPressed = false
    score = 0
    readScore = false
    
    ubuntu1 = love.graphics.newFont("UbuntuMono-B.ttf", 200)
    ubuntu = love.graphics.newFont("UbuntuMono-B.ttf", 40)
    ubuntu2 = love.graphics.newFont("UbuntuMono-B.ttf", 16)
    
    highscore = require("sick")
    highscore.set("hack.this", 10, "Bandung Global Game Jam", 10)
    
    love.keyboard.setTextInput(false)
    name = ""
    shake = 0
    shake2 = 0
end

function removeBullet(bullet)
    if bullet.prev then
        bullet.prev.next = bullet.next
    end
    
    if bullet.next then
        bullet.next.prev = bullet.prev
    end
    
    bullet = nil
end

function addBullet(bullet)
    bulletList.next = bullet
    bullet.prev = bulletList
    bulletList = bullet
end

function newWave()
    plane.wave = plane.wave + 1
    plane.drawWave = true
    plane.waveTimer = 0
    plane.waveRealTimer = 0
end

function love.update(dt)
    prevXPressed = xPressed
    xPressed = love.keyboard.isDown("x")
    
    if game.state == 0 then
        love.keyboard.setTextInput(false)
        if xPressed then
            game.state = 1
            name = ""
        end
    elseif game.state == 1 then
        love.keyboard.setTextInput(false)
        game.startTimer = game.startTimer + dt
        if player.currentAmmo > 0 then
            if xPressed then
                if not player.die then
                    player.shootTime = player.shootTime + dt
                    if player.shootTime >= player.shootInterval then
                        bullet.sound:stop()
                        bullet.sound:play()
                        player.shootTime = player.shootTime - player.shootInterval
                        local bulletDistance = 5
                        local xx = player.x + math.cos(player.angle + math.pi / 2) * bulletDistance
                        local yy = player.y + math.sin(player.angle + math.pi / 2) * bulletDistance
                        addBullet({prev = nil, next = nil, value = {x = xx, y = yy, angle = player.angle, alpha = 0}})
                        xx = player.x + math.cos(player.angle - math.pi / 2) * bulletDistance
                        yy = player.y + math.sin(player.angle - math.pi / 2) * bulletDistance
                        addBullet({prev = nil, next = nil, value = {x = xx, y = yy, angle = player.angle, alpha = 0}})
                        player.currentAmmo = player.currentAmmo - 1
                    end
                end
            elseif love.keyboard.isDown("c") then
                if not player.die then
                    player.shootTime = player.shootTime + dt
                    if player.shootTime >= player.shootInterval then
                        bullet.sound:stop()
                        bullet.sound:play()
                        player.shootTime = player.shootTime - player.shootInterval
                        addBullet({prev = nil, next = nil, value = {x = player.x, y = player.y, angle = player.angle + math.pi / 2, alpha = 0}})
                        addBullet({prev = nil, next = nil, value = {x = player.x, y = player.y, angle = player.angle - math.pi / 2, alpha = 0}})
                        player.currentAmmo = player.currentAmmo - 1
                    end
                end
            end
        else
            player.reloadTimer = player.reloadTimer + dt
            if player.reloadTimer >= player.reloadTime then
                player.currentAmmo = player.maxAmmo
                player.reloadTimer = 0
            end
        end
        
        local left = camera.x
        local right = camera.x + love.graphics.getWidth()
        local top = camera.y
        local bot = camera.y + love.graphics.getHeight()
        
        ship.spawnTimer = ship.spawnTimer + dt
        if ship.spawnTimer >= ship.spawnInterval then
            ship.count = ship.count + 1
            repeat
                ships[ship.count] = {x = love.math.random(0, map.width), y = love.math.random(0, map.height), alpha = 0, waitTime = ship.maxWaitTime, maxWaitTime = ship.maxWaitTime, angle = 0, move = true, radarAlpha = 0}
            until ships[ship.count].x < left or ships[ship.count].x > right or ships[ship.count].y < top or ships[ship.count].y > bot
            ship.spawnTimer = ship.spawnTimer - ship.spawnInterval
            ship.spawnInterval = ship.spawnInterval - ship.spawnIntervalModifier
            if ship.spawnInterval < ship.minSpawnInterval then
                ship.spawnInterval = ship.minSpawnInterval
            end
        end
        
        if plane.count <= 0 then
            print("hure")
            planes = {}
            for i = 1, plane.nextSpawn do
                repeat
                    planes[i] = {x = love.math.random(0, map.width), y = love.math.random(0, map.height), alpha = 255, angle = 0, cooldownTimer = 0, shootTimer = 0, totalShootTimer = 0, shooting = false, radarAlpha = 0}
                until planes[i].x < left or planes[i].x > right or planes[i].y < top or planes[i].y > bot
            end
            plane.count = plane.nextSpawn
            plane.nextSpawn = math.min(plane.nextSpawn + 1, plane.maxSpawn)
            newWave()
            if plane.wave == 1 then
                score = score + score * plane.waveRealTimer / 20
            elseif plane.wave > 1 then
                score = score + score * plane.waveRealTimer / (plane.wave * 10)
            end
        end
        
        plane.waveRealTimer = plane.waveRealTimer + dt
        if plane.wave > 1 and plane.waveRealTimer >= plane.wave * 10 then
            newWave()
        elseif plane.wave == 1 and plane.waveRealTimer >= plane.wave * 20 then
            newWave()
        end
        
        if plane.drawWave then
            plane.waveTimer = plane.waveTimer + dt
            if plane.waveTimer >= plane.waveTime then
                plane.drawWave = false
            end
        end
        
        for i = 1, ship.count do
            ships[i].waitTime = ships[i].waitTime + dt
            if ships[i].waitTime >= ships[i].maxWaitTime then
                xTarget = love.math.random(0, map.width)
                yTarget = love.math.random(0, map.height)
                ships[i].angle = getAngle(ships[i].x, ships[i].y, xTarget, yTarget)
                ships[i].waitTime = ships[i].waitTime - ships[i].maxWaitTime
                ships[i].maxWaitTime = math.random() * ship.maxWaitTime
                
                if ships[i].x == 0 or ships[i].x == map.width or ships[i].y == 0 or ships[i].y == map.height then
                    ships[i].move = true
                elseif (love.math.random(0, 1) == 0) then
                    ships[i].move = true
                else
                    ships[i].move = false
                end
            end
            
            if ships[i].move then
                local enemyMovement = ship.movement * dt
                ships[i].x = ships[i].x - math.cos(ships[i].angle) * enemyMovement
                ships[i].y = ships[i].y - math.sin(ships[i].angle) * enemyMovement
            end
            
            if ships[i].shootTimer then
                ships[i].shootTimer = ships[i].shootTimer + dt
                if ships[i].shootTimer >= ship.shootInterval then
                    if game.startTimer >= game.startTime then
                        ship.bulletCount = ship.bulletCount + 1
                        ship.bullets[ship.bulletCount] = {x = ships[i].x, y = ships[i].y, angle = getAngle(ships[i].x, ships[i].y, player.x, player.y), alpha = 0}
                    end
                    ships[i].shootTimer = ships[i].shootTimer - ship.shootInterval
                end
            end
        end
        
        for i = 1, plane.count do
            local function closestDifference(angleA, angleB)
                local res = angleA - angleB
                res = (res + math.pi) % (math.pi * 2) - math.pi
                return res
            end
            
            local angleToPlayer = getAngle(planes[i].x, planes[i].y, player.x, player.y)
            local angleMovement = plane.turnSpeed * dt
            local angle1 = planes[i].angle + angleMovement
            local angle2 = planes[i].angle - angleMovement
            if closestDifference(angle1, angleToPlayer) < closestDifference(angleToPlayer, angle2) then
                planes[i].angle = angle1
            else
                planes[i].angle = angle2
            end
            
            local planeMovement = plane.speed * dt
            planes[i].x = planes[i].x - math.cos(planes[i].angle) * planeMovement
            planes[i].y = planes[i].y - math.sin(planes[i].angle) * planeMovement
            
            planes[i].cooldownTimer = planes[i].cooldownTimer + dt * plane.shootCooldown
            if not planes[i].shooting then
                if planes[i].cooldownTimer >= plane.shootCooldown and math.abs(angleToPlayer - planes[i].angle) <= 0.375 and math.sqrt((planes[i].x - player.x) ^ 2 + (planes[i].y - player.y) ^ 2) <= 600 then
                    planes[i].shooting = true
                    planes[i].shootTimer = 0
                    planes[i].totalShootTimer = 0
                    planes[i].cooldownTimer = 0
                end
            end
            
            if planes[i].shooting then
                planes[i].shootTimer = planes[i].shootTimer + dt
                if planes[i].shootTimer >= plane.shootInterval then
                    if game.startTimer >= game.startTime then
                        plane.bulletCount = plane.bulletCount + 1
                        plane.bullets[plane.bulletCount] = {x = planes[i].x, y = planes[i].y, angle = planes[i].angle, alpha = 255}
                    end
                    planes[i].shootTimer = planes[i].shootTimer - plane.shootInterval
                end
                
                planes[i].totalShootTimer = planes[i].totalShootTimer + dt
                if planes[i].totalShootTimer >= plane.shootDuration then
                    planes[i].shooting = false
                    planes[i].totalShootTimer = planes[i].totalShootTimer - plane.shootDuration
                    planes[i].cooldownTimer = 0
                end
            end
        end
        
        for i = 1, ship.bulletCount do
            local destroyed = false
            ship.bullets[i].prevX = ship.bullets[i].x
            ship.bullets[i].prevY = ship.bullets[i].y
            
            local bulletMovement = ship.bulletSpeed * dt
            ship.bullets[i].x = ship.bullets[i].x - bulletMovement * math.cos(ship.bullets[i].angle)
            ship.bullets[i].y = ship.bullets[i].y - bulletMovement * math.sin(ship.bullets[i].angle)
            
            local xInterval = (ship.bullets[i].x - ship.bullets[i].prevX) / 3
            local yInterval = (ship.bullets[i].y - ship.bullets[i].prevY) / 3
            for j = 0, 3 do
                local xCur = ship.bullets[i].prevX + j * xInterval
                local yCur = ship.bullets[i].prevY + j * yInterval
                local xDistance = math.abs(xCur - player.x)
                local yDistance = math.abs(yCur - player.y)
                local distance = math.sqrt(xDistance ^ 2 + yDistance ^ 2)
                if distance < player.radius + ship.bulletRadius then
                    addPlayerExplosion(player.x, player.y)
                    player.hitSound:stop()
                    player.hitSound:play()
                    shake = 0.5
                    live.count = math.max(live.count - 5, 0)
                    if live.count <= 0 then
                        game.state = 2
                        player.die = true
                    end
                    ship.bullets[i] = ship.bullets[ship.bulletCount]
                    ship.bulletCount = ship.bulletCount - 1
                    destroyed = true
                    break
                end
            end
            
            if not destroyed then
                if ship.bullets[i].x > right or ship.bullets[i].x < left then
                    if ship.bullets[i].y > bot or ship.bullets[i].y < top then
                        ship.bullets[i] = ship.bullets[ship.bulletCount]
                        ship.bulletCount = ship.bulletCount - 1
                    end
                end
                
                local bulletDistance = math.sqrt((ship.bullets[i].x - player.x) ^ 2 + (ship.bullets[i].y - player.y) ^ 2)
                if bulletDistance <= player.minBulletDetectionRange then
                    ship.bullets[i].alpha = 255
                elseif bulletDistance >= player.maxBulletDetectionRange then
                    ship.bullets[i].alpha = 0
                else
                    local detectionRange = player.maxBulletDetectionRange - player.minBulletDetectionRange
                    ship.bullets[i].alpha = 255 - (((bulletDistance - player.minBulletDetectionRange) / detectionRange) * 255)
                end
            end
        end
        
        for i = 1, plane.bulletCount do
            local destroyed = false
            plane.bullets[i].prevX = plane.bullets[i].x
            plane.bullets[i].prevY = plane.bullets[i].y
            
            local bulletMovement = plane.bulletSpeed * dt
            plane.bullets[i].x = plane.bullets[i].x - bulletMovement * math.cos(plane.bullets[i].angle)
            plane.bullets[i].y = plane.bullets[i].y - bulletMovement * math.sin(plane.bullets[i].angle)
            
            local xInterval = (plane.bullets[i].x - plane.bullets[i].prevX) / 6
            local yInterval = (plane.bullets[i].y - plane.bullets[i].prevY) / 6
            for j = 0, 6 do
                local xCur = plane.bullets[i].prevX + j * xInterval
                local yCur = plane.bullets[i].prevY + j * yInterval
                local xDistance = math.abs(xCur - player.x)
                local yDistance = math.abs(yCur - player.y)
                local distance = math.sqrt(xDistance ^ 2 + yDistance ^ 2)
                if distance < player.radius + plane.bulletRadius then
                    addPlayerExplosion(player.x, player.y)
                    player.hitSound:stop()
                    player.hitSound:play()
                    shake = 0.5
                    live.count = math.max(live.count - 2, 0)
                    if live.count <= 0 then
                        game.state = 2
                        player.die = true
                    end
                    plane.bullets[i] = plane.bullets[plane.bulletCount]
                    plane.bulletCount = plane.bulletCount - 1
                    destroyed = true
                    break
                end
            end
            
            if not destroyed then
                if plane.bullets[i].x > right or plane.bullets[i].x < left then
                    if plane.bullets[i].y > bot or plane.bullets[i].y < top then
                        plane.bullets[i] = plane.bullets[plane.bulletCount]
                        plane.bulletCount = plane.bulletCount - 1
                    end
                end
                
                local bulletDistance = math.sqrt((plane.bullets[i].x - player.x) ^ 2 + (plane.bullets[i].y - player.y) ^ 2)
                if bulletDistance <= player.minBulletDetectionRange then
                    plane.bullets[i].alpha = 255
                elseif bulletDistance >= player.maxBulletDetectionRange then
                    plane.bullets[i].alpha = 0
                else
                    local detectionRange = player.maxBulletDetectionRange - player.minBulletDetectionRange
                    plane.bullets[i].alpha = 255 - (((bulletDistance - player.minBulletDetectionRange) / detectionRange) * 255)
                end
            end
        end
        
        l = bulletList
        while l do
            l.value.prevX = l.value.x
            l.value.prevY = l.value.y
            l.value.x = l.value.x + bullet.speed * math.cos(l.value.angle) * dt
            l.value.y = l.value.y + bullet.speed * math.sin(l.value.angle) * dt
            
            local xInterval = (l.value.x - l.value.prevX) / 12
            local yInterval = (l.value.y - l.value.prevY) / 12
            local removed = false
            
            for i = 1, ship.count do
                for j = 0, 12 do
                    local xCur = l.value.prevX + j * xInterval
                    local yCur = l.value.prevY + j * yInterval
                    local xDistance = math.abs(xCur - ships[i].x)
                    local yDistance = math.abs(yCur - ships[i].y)
                    local distance = math.sqrt(xDistance ^ 2 + yDistance ^ 2)
                    if distance < ship.radius + bullet.radius then
                        enemyHitSound:stop()
                        enemyHitSound:play()
                        addExplosion(ships[i].x, ships[i].y)
                        ships[i] = ships[ship.count]
                        ship.count = ship.count - 1
                        removeBullet(l)
                        score = score + 5
                        removed = true
                        break
                    end
                end
            end
            
            if not removed then
                for i = 1, plane.count do
                    for j = 0, 12 do
                        local xCur = l.value.prevX + j * xInterval
                        local yCur = l.value.prevY + j * yInterval
                        local xDistance = math.abs(xCur - planes[i].x)
                        local yDistance = math.abs(yCur - planes[i].y)
                        local distance = math.sqrt(xDistance ^ 2 + yDistance ^ 2)
                        if distance < plane.radius + bullet.radius then
                            enemyHitSound:stop()
                            enemyHitSound:play()
                            addExplosion(planes[i].x, planes[i].y)
                            planes[i] = planes[plane.count]
                            plane.count = plane.count - 1
                            removeBullet(l)
                            score = score + 10
                            removed = true
                            break
                        end
                    end
                end
                
                if not removed then
                    if l.value.x > right or l.value.x < left then
                        if l.value.y > bot or l.value.y < top then
                            if l.value.x ~= 99999 and l.value.y ~= 99999 then
                                removeBullet(l)
                                removed = true
                            end
                        end
                    end
                    
                    if not removed then
                        local bulletDistance = math.sqrt((l.value.x - player.x) ^ 2 + (l.value.y - player.y) ^ 2)
                        l.value.alpha = 0
                        if bulletDistance <= bullet.minAlphaRange then
                            l.value.alpha = 255
                        elseif bulletDistance >= bullet.maxAlphaRange then
                            l.value.alpha = 0
                        else
                            local detectionRange = bullet.maxAlphaRange - bullet.minAlphaRange
                            l.value.alpha = 255 - (((bulletDistance - bullet.minAlphaRange) / detectionRange) * 255)
                        end
                    end
                end
            end
            
            l = l.prev
        end
        
        player.movement = dt * player.speed
        if love.keyboard.isDown("up") then
            player.x = player.x + math.cos(player.angle) * player.movement
            player.y = player.y + math.sin(player.angle) * player.movement
        end
        if love.keyboard.isDown("left") then
            player.angle = player.angle - dt * player.turnSpeed
        elseif love.keyboard.isDown("right") then
            player.angle = player.angle + dt * player.turnSpeed
        end
        
        for i = 1, ship.count do
            ships[i].radarAngle = getAngle(ships[i].x, ships[i].y, player.x, player.y)
            if ships[i].radarAlpha then
                ships[i].radarAlpha = math.max(ships[i].radarAlpha - dt, 0)
                if radar.angle > ships[i].radarAngle and radar.prevAngle < ships[i].radarAngle then
                    ships[i].radarAlpha = 1
                end
                
                local enemyDistance = math.sqrt((ships[i].x - player.x) ^ 2 + (ships[i].y - player.y) ^ 2)
                local alpha2 = 0
                if enemyDistance <= player.minEnemyDetectionRange then
                    alpha2 = 255
                elseif enemyDistance >= player.maxEnemyDetectionRange then
                    alpha2 = 0
                else
                    local detectionRange = player.maxEnemyDetectionRange - player.minEnemyDetectionRange
                    alpha2 = 255 - (((enemyDistance - player.minEnemyDetectionRange) / detectionRange) * 255)
                end
                
                ships[i].alpha = math.max(ships[i].radarAlpha * 255, alpha2)
            end
        end
        
        for i = 1, plane.count do
            planes[i].radarAngle = getAngle(planes[i].x, planes[i].y, player.x, player.y)
            if planes[i].radarAlpha then
                planes[i].radarAlpha = math.max(planes[i].radarAlpha - dt, 0)
                if radar.angle > planes[i].radarAngle and radar.prevAngle < planes[i].radarAngle then
                    planes[i].radarAlpha = 1
                end
                
                local enemyDistance = math.sqrt((planes[i].x - player.x) ^ 2 + (planes[i].y - player.y) ^ 2)
                local alpha2 = 0
                if enemyDistance <= player.minEnemyDetectionRange then
                    alpha2 = 255
                elseif enemyDistance >= player.maxEnemyDetectionRange then
                    alpha2 = 0
                else
                    local detectionRange = player.maxEnemyDetectionRange - player.minEnemyDetectionRange
                    alpha2 = 255 - (((enemyDistance - player.minEnemyDetectionRange) / detectionRange) * 255)
                end
                
                planes[i].alpha = math.max(planes[i].radarAlpha * 255, alpha2)
            end
        end
        
        if love.keyboard.isDown("r") then
            live.count = 3
            player.die = false
        end
        
    elseif game.state == 2 then
        love.keyboard.setTextInput(true)
        if love.keyboard.isDown("return") then
            highscore.add(name, score)
            highscore.save()
            live.count = live.maxLive
            player.currentAmmo = 10
            game.state = 0
    game.startTime = 5
    game.startTimer = 0
    
    explosion.count = 0
    explosion.image = love.graphics.newImage("ex.png")
    explosion.time = 0.75
    explosion.times = {}
    
    map.width = 2000
    map.height = 1500
    
    bg.image = love.graphics.newImage("bg.png")
    bg.overlayImage = love.graphics.newImage("white.png")
    
    camera.x = 0
    camera.y = 0
    camera.xMax = map.width - love.graphics.getWidth()
    camera.yMax = map.height - love.graphics.getHeight()
    
    radar.angle = 0
    radar.time = 0
    radar.interval = 2
    radar.length = math.sqrt(love.graphics.getWidth() ^ 2 + love.graphics.getHeight() ^ 2)
    radar.trail = {}
    radar.music = love.audio.newSource("radar.wav", "static")
    
    player.x = map.width / 2
    player.y = map.height / 2
    player.angle = 0
    player.speed = 150
    player.turnSpeed = 1
    player.shootInterval = 0.05
    player.shootTime = player.shootInterval
    player.radius = 10
    player.minBulletDetectionRange = 75
    player.maxBulletDetectionRange = 125
    player.minEnemyDetectionRange = 50
    player.maxEnemyDetectionRange = 100
    player.die = false
    player.image = love.graphics.newImage("play.png")
    player.trailCount = 0
    player.trailInterval = 1
    player.trailTimer = 0
    player.maxAmmo = 10
    player.currentAmmo = player.maxAmmo
    player.reloadTime = 3
    player.reloadTimer = 0
    player.lines = {}
    player.explosions = {}
    player.explosionCount = 0
    player.explosionImage = love.graphics.newImage("ex.png")
    player.explosionTime = 0.75
    player.explosionTimes = {}
    player.hitSound = love.audio.newSource("hot.wav", "static")
    
    explodeSound = love.audio.newSource("ex.wav", "static")
    enemyHitSound = love.audio.newSource("hit.wav", "static")
    
    ship.count = 20
    ship.radius = 10
    ship.maxWaitTime = 5;
    ship.movement = 10
    ship.spawnInterval = 5
    ship.minSpawnInterval = 1
    ship.spawnIntervalModifier = 0.5
    ship.spawnTimer = 0
    ship.bulletCount = 0
    ship.shootInterval = 3
    ship.bullets = {}
    ship.bulletSpeed = 225
    ship.bulletRadius = 3
    ship.image = love.graphics.newImage("ship2.png")
    ship.bulletImage = love.graphics.newImage("bul.png")
    for i = 1, ship.count do
        repeat
            ships[i] = {x = love.math.random(0, map.width), y = love.math.random(0, map.height), 
                radarAlpha = 0, waitTime = ship.maxWaitTime, maxWaitTime = ship.maxWaitTime, angle = 0, 
                move = true, shootTimer = love.math.random(0, 5)}
        until math.sqrt((ships[i].x - player.x) ^ 2 + (ships[i].y - player.y) ^ 2) >= 150
    end
    
    plane.count = 0
    plane.wave = 0
    plane.drawWave = false
    plane.waveTime = 1
    plane.waveTimer = 0
    plane.waveRealTime = 0
    plane.nextSpawn = 1
    plane.maxSpawn = 10
    plane.radius = 10
    plane.speed = 100
    plane.turnSpeed = 0.75
    plane.image = love.graphics.newImage("plane2.png")
    plane.bullets = {}
    plane.bulletSpeed = 500
    plane.bulletRadius = 3
    plane.bulletCount = 0
    plane.bulletImage = love.graphics.newImage("eb.png")
    plane.shootDuration = 1
    plane.shootInterval = 0.2
    plane.shootCooldown = 10
    
    bulletList = {prev = nil, next = nil, value = {x = 99999, y = 99999, angle = 0, alpha = 0}}
    bullet.speed = 1000
    bullet.radius = 3
    bullet.minAlphaRange = 300
    bullet.maxAlphaRange = 1500
    bullet.image = love.graphics.newImage("pb.png")
    bullet.sound = love.audio.newSource("shoot.wav", "static")
    
    live.maxLive = 50
    live.count = live.maxLive
    
    ui.leftImage = love.graphics.newImage("ui_l.png")
    ui.rightImage = love.graphics.newImage("ui_r.png")
    ui.middleImage = love.graphics.newImage("ui_m.png")
    
    xPressed = false
    score = 0
    readScore = false
    
    ubuntu1 = love.graphics.newFont("UbuntuMono-B.ttf", 200)
    ubuntu = love.graphics.newFont("UbuntuMono-B.ttf", 40)
    ubuntu2 = love.graphics.newFont("UbuntuMono-B.ttf", 16)
    
    love.keyboard.setTextInput(false)
    name = ""
    shake = 0
    shake2 = 0
            game.state = 0
        end
    end
    
    if love.keyboard.isDown("escape") then
        love.event.push("quit")
    end
    
    radar.time = radar.time + dt
    if radar.time >= radar.interval then
        radar.time = radar.time - radar.interval
        if game.state == 1 then
            radar.music:play()
        end
    end
    radar.prevAngle = radar.angle
    radar.angle = (math.pi * 2) * (radar.time / radar.interval) - math.pi
    radar.x = radar.length * math.cos(radar.angle)
    radar.y = radar.length * math.sin(radar.angle)
    
    radar.trail.angle = radar.angle - 0.5
    radar.trail.x = radar.length * math.cos(radar.trail.angle)
    radar.trail.y = radar.length * math.sin(radar.trail.angle)
    
    local vertices = {
        {
            player.x, player.y,
            0, 0,
            0, 255, 0, 0
        },
        {
            player.x + radar.x, player.y + radar.y,
            0, 0,
            0, 255, 0, 128
        },
        {
            player.x + radar.trail.x, player.y + radar.trail.y,
            0, 0,
            0, 255, 0, 0
        }
    }
    
    mesh = love.graphics.newMesh(vertices)
    
    camera.x = player.x - love.graphics.getHalfWidth()
    camera.x = math.max(camera.x, 0)
    camera.x = math.min(camera.x, camera.xMax)
    camera.y = player.y - love.graphics.getHalfHeight()
    camera.y = math.max(camera.y, 0)
    camera.y = math.min(camera.y, camera.yMax)
    
    for i = 1, explosion.count do
        explosion.times[i] = explosion.times[i] + dt
        if explosion.times[i] >= explosion.time then
            explosions[i] = explosions[explosion.count]
            explosion.times[i] = explosion.times[explosion.count]
            explosion.count = explosion.count - 1
        end
        explosions[i]:setDirection(math.random() * math.pi * 2)
        if explosion.times[i] <= explosion.time / 2 then
            explosions[i]:start()
        end
        explosions[i]:update(dt)
    end
    
    for i = 1, player.explosionCount do
        player.explosionTimes[i] = player.explosionTimes[i] + dt
        if player.explosionTimes[i] >= player.explosionTime then
            player.explosions[i] = player.explosions[player.explosionCount]
            player.explosionTimes[i] = player.explosionTimes[player.explosionCount]
            player.explosionCount = player.explosionCount - 1
        end
        player.explosions[i]:setDirection(math.random() * math.pi * 2)
        player.explosions[i]:setPosition(player.x, player.y)
        if player.explosionTimes[i] <= player.explosionTime / 2 then
            player.explosions[i]:start()
        end
        player.explosions[i]:update(dt)
    end
    
    shake = shake - dt
    shake2 = shake2 - dt
end

function love.textinput(t)
    name = name..t
end

function isXReleased()
    return prevXPressed == true and xPressed == false
end

function getAngle(x1, y1, x2, y2)
    local deltaX = x1 - x2
    local deltaY = y1 - y2
    return math.atan2(deltaY, deltaX)
end

function addExplosion(xx, yy)
    explosion.count = explosion.count + 1
    explosion.times[explosion.count] = 0
    explosions[explosion.count] = love.graphics.newParticleSystem(explosion.image, 256)
    explosions[explosion.count]:setEmissionRate(40)
    explosions[explosion.count]:setEmitterLifetime(0.5)
    explosions[explosion.count]:setParticleLifetime(1)
    explosions[explosion.count]:setPosition(xx, yy)
    explosions[explosion.count]:setDirection(0)
    explosions[explosion.count]:setSpread(math.pi * 2)
    explosions[explosion.count]:setSpeed(20, 60)
    explosions[explosion.count]:setLinearAcceleration(0, 0)
    explosions[explosion.count]:setRadialAcceleration(10)
    explosions[explosion.count]:setTangentialAcceleration(0)
    explosions[explosion.count]:setSizes(2, 1)
    explosions[explosion.count]:setSizeVariation(0.25)
    explosions[explosion.count]:setRotation(0)
    explosions[explosion.count]:setSpin(0)
    explosions[explosion.count]:setSpinVariation(0)
    explosions[explosion.count]:setColors(25, 245, 25, 128, 175, 175, 175, 128, 175, 175, 175, 0, 175, 175, 175, 0)
    explosions[explosion.count]:stop()
end

function addPlayerExplosion(xx, yy)
    player.explosionCount = player.explosionCount + 1
    player.explosionTimes[player.explosionCount] = 0
    player.explosions[player.explosionCount] = love.graphics.newParticleSystem(explosion.image, 256)
    player.explosions[player.explosionCount]:setEmissionRate(40)
    player.explosions[player.explosionCount]:setEmitterLifetime(0.5)
    player.explosions[player.explosionCount]:setParticleLifetime(1)
    player.explosions[player.explosionCount]:setPosition(xx, yy)
    player.explosions[player.explosionCount]:setDirection(0)
    player.explosions[player.explosionCount]:setSpread(math.pi * 2)
    player.explosions[player.explosionCount]:setSpeed(20, 60)
    player.explosions[player.explosionCount]:setLinearAcceleration(0, 0)
    player.explosions[player.explosionCount]:setRadialAcceleration(10)
    player.explosions[player.explosionCount]:setTangentialAcceleration(0)
    player.explosions[player.explosionCount]:setSizes(2, 1)
    player.explosions[player.explosionCount]:setSizeVariation(0.25)
    player.explosions[player.explosionCount]:setRotation(0)
    player.explosions[player.explosionCount]:setSpin(0)
    player.explosions[player.explosionCount]:setSpinVariation(0)
    player.explosions[player.explosionCount]:setColors(255, 0, 25, 128, 175, 175, 175, 128, 175, 175, 175, 0, 175, 175, 175, 0)
    player.explosions[player.explosionCount]:stop()
end

function love.draw()
    love.graphics.push()
    love.graphics.translate(-camera.x, -camera.y)
    if shake > 0 then
        love.graphics.translate(love.math.random(1, 18) - 9, love.math.random(1, 18) - 9)
    end
    if shake2 > 0 then
        love.graphics.translate(love.math.random(1, 4) - 2, love.math.random(1, 4) - 2)
    end
    love.graphics.setColor(0, 0, 0)
    love.graphics.draw(bg.image, camera.x, camera.y, 0, love.graphics.getWidth() / bg.image:getWidth(), love.graphics.getHeight() / bg.image:getHeight())
    
    love.graphics.setLineWidth(1)
    for i = 0, map.width, 40 do
        love.graphics.setColor(0, 255, 25, 28 + love.math.random(0, 40))
        love.graphics.line(i, 0, i, map.height)
    end
    for i = 0, map.height, 40 do
        love.graphics.setColor(0, 255, 25, 28 + love.math.random(0, 40))
        love.graphics.line(0, i, map.width, i)
    end
    
    if game.state == 0 then
        love.graphics.setColor(25, 245, 25, 100 + love.math.random(0, 155))
        love.graphics.setFont(ubuntu1)
        love.graphics.printf("VANISH", camera.x, camera.y + 100, love.graphics.getWidth(), 'center')
        love.graphics.setFont(ubuntu2)
        love.graphics.printf("CONTROLS\n\nmove with [up]\nturn left with [left]\nturn right with [right]\nfire forward with [x]\nfire sideways [c]\nquit game with [escape]\n\ncomplete waves early to obtain more points!", camera.x, camera.y + love.graphics.getHeight() / 2 - 40, love.graphics.getWidth() / 2 - 10, 'right')
        local hText = "HIGHSCORES\n\n"
        for i, scor, name in highscore() do
            hText = hText..name.." - "..scor.."\n"
        end
        love.graphics.printf(hText, camera.x + love.graphics.getWidth() / 2 + 10, camera.y + love.graphics.getHeight() / 2 - 40, love.graphics.getWidth() / 2 - 10, 'left')
        love.graphics.line(camera.x + love.graphics.getWidth() / 2, camera.y + love.graphics.getHeight() / 2 - 50, camera.x + love.graphics.getWidth() / 2, camera.y + love.graphics.getHeight() / 2 + 162)
    elseif game.state == 1 then
        love.graphics.setColor(0, 250, 0, 128)
        love.graphics.setLineWidth(3)
        love.graphics.line(player.x, player.y, player.x + radar.x, player.y + radar.y)
        love.graphics.draw(mesh, 0, 0)
        
        for i = 1, ship.count do
            love.graphics.setColor(255, 255, 255, ships[i].alpha)
            love.graphics.draw(ship.image, ships[i].x, ships[i].y, ships[i].angle + math.pi, 1, 1, ship.image:getWidth() / 2, ship.image:getHeight() / 2)
        end
        
        for i = 1, plane.count do
            love.graphics.setColor(255, 255, 255, planes[i].alpha)
            love.graphics.draw(plane.image, planes[i].x, planes[i].y, planes[i].angle + math.pi, 1, 1, plane.image:getWidth() / 2, plane.image:getHeight() / 2)
        end
        
        love.graphics.setColor(255, 255, 255, 255)
        love.graphics.draw(player.image, player.x, player.y, player.angle, 1, 1, player.image:getWidth() / 2, player.image:getHeight() / 2)
        
        l = bulletList
        while l do
            love.graphics.setColor(255, 255, 255, l.value.alpha)
            love.graphics.draw(bullet.image, l.value.x, l.value.y, l.value.angle, 1, 1, bullet.image:getWidth() / 2, bullet.image:getHeight() / 2)
            l = l.prev
        end
        
        for i = 1, ship.bulletCount do
            love.graphics.setColor(255, 255, 255, ship.bullets[i].alpha)
            love.graphics.draw(ship.bulletImage, ship.bullets[i].x, ship.bullets[i].y, 0, 1, 1, ship.bulletImage:getWidth() / 2, ship.bulletImage:getHeight() / 2)
        end
        
        for i = 1, plane.bulletCount do
            love.graphics.setColor(255, 255, 255, plane.bullets[i].alpha)
            love.graphics.draw(plane.bulletImage, plane.bullets[i].x, plane.bullets[i].y, 0, 1, 1, plane.bulletImage:getWidth() / 2, plane.bulletImage:getHeight() / 2)
        end
        
        love.graphics.setColor(255, 255, 255, 255)
        love.graphics.draw(ui.leftImage, camera.x, camera.y)
        love.graphics.draw(ui.middleImage, love.graphics.newQuad(0, 0, live.count / live.maxLive * (love.graphics.getWidth() - 72), 19, 19, 19), camera.x + 52, camera.y + 2)
        love.graphics.draw(ui.middleImage, love.graphics.newQuad(0, 0, player.currentAmmo / player.maxAmmo * (love.graphics.getWidth() - 72), 19, 19, 19), camera.x + 52, camera.y + 18)
        --love.graphics.draw(ui.rightImage, camera.x + love.graphics.getWidth() - 20, camera.y + 2)
        
        love.graphics.setColor(255, 255, 255, 255)
        for i = 1, explosion.count do
            love.graphics.draw(explosions[i], 0, 0)
        end
        
        love.graphics.setColor(255, 0, 25, 255)
        for i = 1, player.explosionCount do
            love.graphics.draw(player.explosions[i], 0, 0)
        end
        
        if plane.drawWave then
            love.graphics.setColor(25, 245, 25, 64)
            love.graphics.rectangle("fill", camera.x, camera.y + love.graphics.getHeight() / 2 - 25, love.graphics.getWidth(), 50)
            love.graphics.setFont(ubuntu)
            love.graphics.setColor(255, 255, 255, 255)
            love.graphics.printf("WAVE "..tostring(plane.wave), camera.x, camera.y + love.graphics.getHeight() / 2 - 20, love.graphics.getWidth(), 'center')
        end
        
        love.graphics.setColor(255, 255, 255, 255)
        love.graphics.setFont(ubuntu2)
        love.graphics.printf(tostring(score), camera.x, camera.y + love.graphics.getHeight() - 18, love.graphics.getWidth(), 'center')
    elseif game.state == 2 then
        love.graphics.setColor(25, 245, 25, 150 + love.math.random(0, 105))
        love.graphics.setFont(ubuntu2)
        local high = 0
        local textScore = "YOUR SCORE IS "..tostring(score)
        for i, scor, name in highscore() do
            if scor >= score then
                high = high + 1
            end
        end
        if high < 10 then
            textScore = textScore.." - NEW HIGHSCORE!"
        end
        love.graphics.printf(textScore, camera.x, camera.y + love.graphics.getHeight() / 2 - 21, love.graphics.getWidth(), 'center')
        local nameText = "YOUR NAME : "..name
        love.graphics.printf(nameText, camera.x, camera.y + love.graphics.getHeight() / 2 + 5, love.graphics.getWidth(), 'center')
    end
    
    love.graphics.setColor(255, 255, 255, 32)
    love.graphics.draw(bg.overlayImage, camera.x, camera.y + love.math.random(4) - 2, 0, love.graphics.getWidth() / bg.overlayImage:getWidth(), love.graphics.getHeight() / bg.overlayImage:getHeight())
    love.graphics.pop()
end

function love.graphics.getHalfHeight()
    return love.graphics.getHeight() / 2
end

function love.graphics.getHalfWidth()
    return love.graphics.getWidth() / 2
end